@@ -206,5 +206,17 @@ T StringBuffer_clear(T S);
 const char *StringBuffer_toString(T S);
 
 
+/**
+ * Returns the content of this string buffer as gzip compressed data (binary data).
+ * @param S StringBuffer object
+ * @param level compression level. A number between 0 and 9 where 1 gives
+ * best speed, 9 gives best compression, 0 gives no compression. 6 is a good value.
+ * @param length The number of bytes in the returned data is stored in length
+ * @return The compressed data representing this string buffer 
+ * @exception AssertException if level is not in [0..9] or if compression failed
+ */
+const void *StringBuffer_toCompressed(T S, int level, size_t *length);
+
+
 #undef T
 #endif
