@@ -110,6 +110,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
         }
         END_TRY;
         if (C) {
+                int64_t _timeoutMilli = *timeout / 1000.;
                 for (int i = 1; i < c->length; i++)
                         Command_appendArgument(C, c->arg[i]);
                 if (c->has_uid)
@@ -142,7 +143,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
                                 *timeout -= RETRY_INTERVAL;
                         } while ((status = Process_exitStatus(P)) < 0 && *timeout > 0 && ! (Run.flags & Run_Stopped));
                         if (*timeout <= 0)
-                                snprintf(msg, msglen, "Program %s timed out", c->arg[0]);
+                                snprintf(msg, msglen, "Program '%s' timed out after %s", Util_commandDescription(c, (char[STRLEN]){}), Str_milliToTime(_timeoutMilli, (char[23]){}));
                         int n, total = 0;
                         char buf[STRLEN];
                         do {
@@ -153,7 +154,7 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, int6
                                         DEBUG("%s", buf);
                                         // Report the first message (override existing plain timeout message if some program output is available)
                                         if (! total)
-                                                snprintf(msg, msglen, "%s: %s%s", c->arg[0], *timeout <= 0 ? "Program timed out -- " : "", buf);
+                                                snprintf(msg, msglen, "'%s': %s%s", Util_commandDescription(c, (char[STRLEN]){}), *timeout <= 0 ? "Program timed out -- " : "", buf);
                                         total += n;
                                 }
                         } while (n > 0 && Run.debug && total < 2048); // Limit the debug output (if the program will have endless output, such as 'yes' utility, we have to stop at some point to not spin here forever)
@@ -201,7 +202,7 @@ static State_Type _check(Service_T s) {
         rv = s->check(s);
         if (s->type == Service_Program) {
                 // check program executes the program and needs to be called again to collect the exit value and evaluate the status
-                int64_t timeout = s->program->timeout * 1000000;
+                int64_t timeout = s->program->timeout * USEC_PER_MSEC;
                 do {
                         Time_usleep(RETRY_INTERVAL);
                         timeout -= RETRY_INTERVAL;
@@ -239,9 +240,9 @@ static boolean_t _doStart(Service_T s) {
         if (rv) {
                 if (s->start) {
                         if (s->type != Service_Process || ! ProcessTree_findProcess(s)) {
-                                LogInfo("'%s' start: %s\n", s->name, s->start->arg[0]);
+                                LogInfo("'%s' start: '%s'\n", s->name, Util_commandDescription(s->start, (char[STRLEN]){}));
                                 char msg[STRLEN];
-                                int64_t timeout = s->start->timeout * USEC_PER_SEC;
+                                int64_t timeout = s->start->timeout * USEC_PER_MSEC;
                                 int status = _commandExecute(s, s->start, msg, sizeof(msg), &timeout);
                                 if (status < 0 || (s->type == Service_Process && _waitProcessStart(s, &timeout) != Process_Started)) {
                                         Event_post(s, Event_Exec, State_Failed, s->action_EXEC, "failed to start (exit status %d) -- %s", status, *msg ? msg : "no output");
@@ -265,7 +266,7 @@ static boolean_t _doStart(Service_T s) {
 
 
 static int _executeStop(Service_T s, char *msg, int msglen, int64_t *timeout) {
-        LogInfo("'%s' stop: %s\n", s->name, s->stop->arg[0]);
+        LogInfo("'%s' stop: '%s'\n", s->name, Util_commandDescription(s->stop, (char[STRLEN]){}));
         return _commandExecute(s, s->stop, msg, msglen, timeout);
 }
 
@@ -291,7 +292,7 @@ static boolean_t _doStop(Service_T s, boolean_t unmonitor) {
                 if (s->monitor != Monitor_Not) {
                         int exitStatus;
                         char msg[STRLEN];
-                        int64_t timeout = s->stop->timeout * USEC_PER_SEC;
+                        int64_t timeout = s->stop->timeout * USEC_PER_MSEC;
                         if (s->type == Service_Process) {
                                 int pid = ProcessTree_findProcess(s);
                                 if (pid) {
@@ -326,10 +327,10 @@ static boolean_t _doRestart(Service_T s) {
         ASSERT(s);
         boolean_t rv = true;
         if (s->restart) {
-                LogInfo("'%s' restart: %s\n", s->name, s->restart->arg[0]);
+                LogInfo("'%s' restart: '%s'\n", s->name, Util_commandDescription(s->restart, (char[STRLEN]){}));
                 Util_resetInfo(s);
                 char msg[STRLEN];
-                int64_t timeout = s->restart->timeout * USEC_PER_SEC;
+                int64_t timeout = s->restart->timeout * USEC_PER_MSEC;
                 int status = _commandExecute(s, s->restart, msg, sizeof(msg), &timeout);
                 if (status < 0 || (s->type == Service_Process && _waitProcessStart(s, &timeout) != Process_Started)) {
                         rv = false;
