@@ -91,8 +91,8 @@ static volatile boolean_t running = false;
  */
 boolean_t can_http() {
         if ((Run.httpd.flags & Httpd_Net || Run.httpd.flags & Httpd_Unix) && (Run.flags & Run_Daemon)) {
-                if (! Engine_hasHostsAllow() && ! Run.httpd.credentials && ! ((Run.httpd.flags & Httpd_Ssl) && (Run.httpd.flags & Httpd_Net) && Run.httpd.socket.net.ssl.clientpem)) {
-                        LogError("%s: monit httpd not started since no connect allowed\n", prog);
+                if (! Engine_hasAllow() && ! Run.httpd.credentials && ! ((Run.httpd.flags & Httpd_Ssl) && (Run.httpd.flags & Httpd_Net) && Run.httpd.socket.net.ssl.clientpem)) {
+                        LogError("%s: monit httpd not started since no connections are allowed\n", prog);
                         return false;
 
                 }
