@@ -89,6 +89,7 @@ typedef struct response {
         HttpHeader headers;
         const char *status_msg;
         StringBuffer_T outputbuffer;
+        MD_T token;
         Ssl_T ssl;
 } *HttpResponse;
 
@@ -102,9 +103,9 @@ void add_Impl(void(*doGet)(HttpRequest, HttpResponse), void(*doPost)(HttpRequest
 void set_content_type(HttpResponse res, const char *mime);
 const char *get_header(HttpRequest req, const char *header_name);
 void escapeHTML(StringBuffer_T sb, const char *s);
-void send_error(HttpRequest, HttpResponse, int status, const char *message, ...);
+void send_error(HttpRequest, HttpResponse, int status, const char *message, ...) __attribute__((format (printf, 4, 5)));
 const char *get_parameter(HttpRequest req, const char *parameter_name);
-void set_header(HttpResponse res, const char *name, const char *value);
+void set_header(HttpResponse res, const char *name, const char *value, ...) __attribute__((format (printf, 3, 4)));
 void Processor_setHttpPostLimit();
 
 #endif
