@@ -38,24 +38,18 @@
 #include "exceptions/IOException.h"
 
 /* Escape zero i.e. '\0' in expect buffer with "\0" so zero can be tested in expect strings as "\0". If there are no '\0' in the buffer it is returned as it is */
-static char *_escapeZeroInExpectBuffer(char *s, int n) {
-        assert(n <= Run.limits.sendExpectBuffer);
-        int i, j;
-        char t[n]; // VLA
-        for (i = 0, j = 0; j < n; i++, j++) {
-                if ((t[j] = s[i]) == '\0') {
-                        if (j + 2 < n) {
-                                t[j] = '\\';
-                                t[j + 1] = '0';
+static char *_escapeZeroInExpectBuffer(char *buf, int buflen, int n) {
+        for (int i = 0, j = 0; i < n && j < buflen; i++, j++) {
+                if (buf[j] == '\0') {
+                        if (j + 1 < buflen) {
+                                memmove(buf + j + 1, buf + j, n - i);
+                                buf[j] = '\\';
+                                buf[j + 1] = '0';
                                 j++;
                         }
                 }
         }
-        if (j > i) {
-                memcpy(s, t, n);
-                s[n] = 0;
-        }
-        return s;
+        return buf;
 }
 
 
@@ -105,7 +99,7 @@ void check_generic(Socket_T socket) {
                         int n = Socket_read(socket, buf + 1, Run.limits.sendExpectBuffer - 1) + 1;
                         buf[n] = 0;
                         if (n > 0)
-                                _escapeZeroInExpectBuffer(buf, n);
+                                _escapeZeroInExpectBuffer(buf, Run.limits.sendExpectBuffer, n);
                         Socket_setTimeout(socket, timeout); // Reset back original timeout for next send/expect
                         int regex_return = regexec(g->expect, buf, 0, NULL, 0);
                         if (regex_return != 0) {
