@@ -158,7 +158,7 @@ static int _fill(T S, int timeout) {
 }
 
 
-int _getPort(const struct sockaddr *addr, socklen_t addrlen) {
+int _getPort(const struct sockaddr *addr) {
         if (addr->sa_family == AF_INET)
                 return ntohs(((struct sockaddr_in *)addr)->sin_port);
 #ifdef HAVE_IPV6
@@ -246,7 +246,7 @@ T _createIpSocket(const char *host, const struct sockaddr *addr, socklen_t addrl
                                         S->family = family == AF_INET ? Socket_Ip4 : Socket_Ip6;
                                         S->timeout = timeout;
                                         S->host = Str_dup(host);
-                                        S->port = _getPort(addr, addrlen);
+                                        S->port = _getPort(addr);
                                         S->connection_type = Connection_Client;
                                         if (ssl.flags == SSL_Enabled) {
                                                 TRY
@@ -314,38 +314,6 @@ struct addrinfo *_resolve(const char *hostname, int port, Socket_Type type, Sock
 }
 
 
-static T _createUnixSocket(const char *pathname, Socket_Type type, int timeout) {
-        struct sockaddr_un unixsocket;
-        ASSERT(pathname);
-        int s = socket(PF_UNIX, type, 0);
-        if (s >= 0) {
-                unixsocket.sun_family = AF_UNIX;
-                snprintf(unixsocket.sun_path, sizeof(unixsocket.sun_path), "%s", pathname);
-                if (Net_setNonBlocking(s)) {
-                        char error[STRLEN];
-                        if (_doConnect(s, (struct sockaddr *)&unixsocket, sizeof(unixsocket), timeout, error, sizeof(error))) {
-                                T S;
-                                NEW(S);
-                                S->connection_type = Connection_Client;
-                                S->family = Socket_Unix;
-                                S->type = type;
-                                S->socket = s;
-                                S->timeout = timeout;
-                                S->host = Str_dup(LOCALHOST);
-                                return S;
-                        }
-                        LogError("Unix socket %s error -- %s\n", pathname, error);
-                } else {
-                        LogError("Cannot set nonblocking unix socket %s -- %s\n", pathname, STRERROR);
-                }
-                Net_close(s);
-        } else {
-                LogError("Cannot create unix socket %s -- %s\n", pathname, STRERROR);
-        }
-        return NULL;
-}
-
-
 /* ------------------------------------------------------------------ Public */
 
 
@@ -384,11 +352,49 @@ T Socket_create(const char *host, int port, Socket_Type type, Socket_Family fami
 T Socket_createUnix(const char *path, Socket_Type type, int timeout) {
         ASSERT(path);
         ASSERT(timeout > 0);
-        return _createUnixSocket(path, type, timeout);
+        int s = socket(PF_UNIX, type, 0);
+        if (s >= 0) {
+                struct sockaddr_un unixsocket_client = {};
+                if (type == Socket_Udp) {
+                        unixsocket_client.sun_family = AF_UNIX;
+                        snprintf(unixsocket_client.sun_path, sizeof(unixsocket_client.sun_path), "/tmp/monit_%llx.sock", (long long unsigned)&unixsocket_client);
+                        if (bind(s, (struct sockaddr *) &unixsocket_client, sizeof(unixsocket_client)) != 0) {
+                                LogError("Unix socket %s bind error -- %s\n", unixsocket_client.sun_path, STRERROR);
+                                goto error;
+                        }
+                }
+                struct sockaddr_un unixsocket_server = {};
+                unixsocket_server.sun_family = AF_UNIX;
+                strncpy(unixsocket_server.sun_path, path, sizeof(unixsocket_server.sun_path));
+                if (Net_setNonBlocking(s)) {
+                        char error[STRLEN];
+                        if (_doConnect(s, (struct sockaddr *)&unixsocket_server, sizeof(unixsocket_server), timeout, error, sizeof(error))) {
+                                T S;
+                                NEW(S);
+                                S->connection_type = Connection_Client;
+                                S->family = Socket_Unix;
+                                S->type = type;
+                                S->socket = s;
+                                S->timeout = timeout;
+                                S->host = Str_dup(LOCALHOST);
+                                return S;
+                        }
+                        LogError("Unix socket %s connection error -- %s\n", path, error);
+                } else {
+                        LogError("Cannot set nonblocking unix socket %s -- %s\n", path, STRERROR);
+                }
+error:
+                Net_close(s);
+                if (type == Socket_Udp)
+                        unlink(unixsocket_client.sun_path);
+        } else {
+                LogError("Cannot create unix socket %s -- %s\n", path, STRERROR);
+        }
+        return NULL;
 }
 
 
-T Socket_createAccepted(int socket, struct sockaddr *addr, socklen_t addrlen, void *sslserver) {
+T Socket_createAccepted(int socket, struct sockaddr *addr, void *sslserver) {
         ASSERT(socket >= 0);
         ASSERT(addr);
         T S;
@@ -410,7 +416,7 @@ T Socket_createAccepted(int socket, struct sockaddr *addr, socklen_t addrlen, vo
                         S->host = Str_dup(inet_ntop(addr->sa_family, &a->sin6_addr, (char[INET6_ADDRSTRLEN]){}, INET6_ADDRSTRLEN));
                 }
 #endif
-                S->port = _getPort(addr, addrlen);
+                S->port = _getPort(addr);
 #ifdef HAVE_OPENSSL
                 if (sslserver) {
                         S->sslserver = sslserver;
@@ -442,6 +448,19 @@ void Socket_free(T *S) {
         else
 #endif
         {
+                int type;
+                socklen_t length = sizeof(type);
+                getsockopt((*S)->socket, SOL_SOCKET, SO_TYPE, &type, &length);
+                if (type == SOCK_DGRAM) {
+                        struct sockaddr_storage addr;
+                        socklen_t addrlen = sizeof(addr);
+                        if (getsockname((*S)->socket, (struct sockaddr *)&addr, &addrlen) == 0) {
+                                if (addr.ss_family == AF_UNIX) {
+                                        struct sockaddr_un *_addr = (struct sockaddr_un *)&addr;
+                                        unlink(_addr->sun_path);
+                                }
+                        }
+                }
                 Net_shutdown((*S)->socket, SHUT_RDWR);
                 Net_close((*S)->socket);
         }
@@ -510,7 +529,7 @@ int Socket_getLocalPort(T S) {
         struct sockaddr_storage addr;
         socklen_t addrlen = sizeof(addr);
         if (getsockname(S->socket, (struct sockaddr *)&addr, &addrlen) == 0)
-                return _getPort((struct sockaddr *)&addr, addrlen);
+                return _getPort((struct sockaddr *)&addr);
         return -1;
 }
 
@@ -534,7 +553,7 @@ const char *Socket_getLocalHost(T S, char *host, int hostlen) {
 
 
 static void _testUnix(Port_T p) {
-        T S = _createUnixSocket(p->target.unix.pathname, p->type, p->timeout);
+        T S = Socket_createUnix(p->target.unix.pathname, p->type, p->timeout);
         if (S) {
                 S->Port = p;
                 TRY
