@@ -147,18 +147,17 @@ struct SslServer_T {
 };
 
 
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
 static Mutex_T *instanceMutexTable;
+#endif
+
+
 static int session_id_context = 1;
 
 
 /* ----------------------------------------------------------------- Private */
 
 
-static unsigned long _threadID() {
-        return (unsigned long)Thread_self();
-}
-
-
 static boolean_t _retry(int socket, int *timeout, int (*callback)(int socket, time_t milliseconds)) {
         long long start = Time_milli();
         if (callback(socket, *timeout)) {
@@ -170,12 +169,19 @@ static boolean_t _retry(int socket, int *timeout, int (*callback)(int socket, ti
 }
 
 
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
+static unsigned long _threadID() {
+        return (unsigned long)Thread_self();
+}
+
+
 static void _mutexLock(int mode, int n, const char *file, int line) {
         if (mode & CRYPTO_LOCK)
                 Mutex_lock(instanceMutexTable[n]);
         else
                 Mutex_unlock(instanceMutexTable[n]);
 }
+#endif
 
 
 static int _checkExpiration(T C, X509_STORE_CTX *ctx, X509 *certificate) {
@@ -311,12 +317,23 @@ static int _verifyClientCertificates(int preverify_ok, X509_STORE_CTX *ctx) {
                                 return 0;
                 }
         }
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         X509_OBJECT found_cert;
         if (X509_STORE_CTX_get_error_depth(ctx) == 0 && X509_STORE_get_by_subject(ctx, X509_LU_X509, X509_get_subject_name(X509_STORE_CTX_get_current_cert(ctx)), &found_cert) != 1) {
+#else
+        X509_OBJECT *found_cert = X509_OBJECT_new();
+        if (X509_STORE_CTX_get_error_depth(ctx) == 0 && X509_STORE_get_by_subject(ctx, X509_LU_X509, X509_get_subject_name(X509_STORE_CTX_get_current_cert(ctx)), found_cert) != 1) {
+#endif
                 LogError("SSL: no matching certificate found -- %s\n", SSLERROR);
                 X509_STORE_CTX_set_error(ctx, X509_V_ERR_CERT_REJECTED);
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && ! defined(LIBRESSL_VERSION_NUMBER)
+                X509_OBJECT_free(found_cert);
+#endif
                 return 0;
         }
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && ! defined(LIBRESSL_VERSION_NUMBER)
+        X509_OBJECT_free(found_cert);
+#endif
         return 1;
 }
 
@@ -360,24 +377,27 @@ static boolean_t _setClientCertificate(T C, const char *file) {
 
 
 void Ssl_start() {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         SSL_library_init();
         SSL_load_error_strings();
-        if (File_exist(URANDOM_DEVICE))
-                RAND_load_file(URANDOM_DEVICE, RANDOM_BYTES);
-        else if (File_exist(RANDOM_DEVICE))
-                RAND_load_file(RANDOM_DEVICE, RANDOM_BYTES);
-        else
-                THROW(AssertException, "SSL: cannot find %s nor %s on the system", URANDOM_DEVICE, RANDOM_DEVICE);
         int locks = CRYPTO_num_locks();
         instanceMutexTable = CALLOC(locks, sizeof(Mutex_T));
         for (int i = 0; i < locks; i++)
                 Mutex_init(instanceMutexTable[i]);
         CRYPTO_set_id_callback(_threadID);
         CRYPTO_set_locking_callback(_mutexLock);
+#endif
+        if (File_exist(URANDOM_DEVICE))
+                RAND_load_file(URANDOM_DEVICE, RANDOM_BYTES);
+        else if (File_exist(RANDOM_DEVICE))
+                RAND_load_file(RANDOM_DEVICE, RANDOM_BYTES);
+        else
+                THROW(AssertException, "SSL: cannot find %s nor %s on the system", URANDOM_DEVICE, RANDOM_DEVICE);
 }
 
 
 void Ssl_stop() {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         CRYPTO_set_id_callback(NULL);
         CRYPTO_set_locking_callback(NULL);
         for (int i = 0; i < CRYPTO_num_locks(); i++)
@@ -385,12 +405,15 @@ void Ssl_stop() {
         FREE(instanceMutexTable);
         RAND_cleanup();
         ERR_free_strings();
+#endif
         Ssl_threadCleanup();
 }
 
 
 void Ssl_threadCleanup() {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         ERR_remove_state(0);
+#endif
 }
 
 
@@ -408,10 +431,11 @@ T Ssl_new(Ssl_Version version, const char *CACertificateFile, const char *CACert
         T C;
         NEW(C);
         C->version = version;
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         const SSL_METHOD *method;
         switch (version) {
                 case SSL_V2:
-#ifdef OPENSSL_NO_SSL2
+#if defined OPENSSL_NO_SSL2 || ! defined HAVE_SSLV2
                         LogError("SSL: SSLv2 not supported\n");
                         goto sslerror;
 #else
@@ -420,10 +444,10 @@ T Ssl_new(Ssl_Version version, const char *CACertificateFile, const char *CACert
                                 goto sslerror;
                         }
                         method = SSLv2_client_method();
-#endif
                         break;
+#endif
                 case SSL_V3:
-#ifdef OPENSSL_NO_SSL3
+#if defined OPENSSL_NO_SSL3
                         LogError("SSL: SSLv3 not supported\n");
                         goto sslerror;
 #else
@@ -432,26 +456,41 @@ T Ssl_new(Ssl_Version version, const char *CACertificateFile, const char *CACert
                                 goto sslerror;
                         }
                         method = SSLv3_client_method();
-#endif
                         break;
+#endif
                 case SSL_TLSV1:
+#if defined OPENSSL_NO_TLS1_METHOD
+                        LogError("SSL: TLSv1.0 not supported\n");
+                        goto sslerror;
+#else
                         method = TLSv1_client_method();
                         break;
-#ifdef HAVE_TLSV1_1
+#endif
                 case SSL_TLSV11:
+#if defined OPENSSL_NO_TLS1_1_METHOD || ! defined HAVE_TLSV1_1
+                        LogError("SSL: TLSv1.1 not supported\n");
+                        goto sslerror;
+#else
                         method = TLSv1_1_client_method();
                         break;
 #endif
-#ifdef HAVE_TLSV1_2
                 case SSL_TLSV12:
+#if defined OPENSSL_NO_TLS1_2_METHOD || ! defined HAVE_TLSV1_2
+                        LogError("SSL: TLSv1.2 not supported\n");
+                        goto sslerror;
+#else
                         method = TLSv1_2_client_method();
                         break;
 #endif
                 case SSL_Auto:
                 default:
                         method = SSLv23_client_method();
+                        //FIXME: add new "set ssl" option for setting minimum version and use SSL_CTX_set_options() to disable protocols
                         break;
         }
+#else
+        const SSL_METHOD *method = TLS_client_method();
+#endif
         if (! method) {
                 LogError("SSL: client method initialization failed -- %s\n", SSLERROR);
                 goto sslerror;
@@ -460,6 +499,33 @@ T Ssl_new(Ssl_Version version, const char *CACertificateFile, const char *CACert
                 LogError("SSL: client context initialization failed -- %s\n", SSLERROR);
                 goto sslerror;
         }
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && ! defined(LIBRESSL_VERSION_NUMBER)
+        switch (version) {
+                case SSL_V2:
+                        LogError("SSL: SSLv2 not supported\n");
+                        goto sslerror;
+                case SSL_V3:
+                        SSL_CTX_set_min_proto_version(C->ctx, SSL3_VERSION);
+                        SSL_CTX_set_max_proto_version(C->ctx, SSL3_VERSION);
+                        break;
+                case SSL_TLSV1:
+                        SSL_CTX_set_min_proto_version(C->ctx, TLS1_VERSION);
+                        SSL_CTX_set_max_proto_version(C->ctx, TLS1_VERSION);
+                        break;
+                case SSL_TLSV11:
+                        SSL_CTX_set_min_proto_version(C->ctx, TLS1_1_VERSION);
+                        SSL_CTX_set_max_proto_version(C->ctx, TLS1_1_VERSION);
+                        break;
+                case SSL_TLSV12:
+                        SSL_CTX_set_min_proto_version(C->ctx, TLS1_2_VERSION);
+                        SSL_CTX_set_max_proto_version(C->ctx, TLS1_2_VERSION);
+                        break;
+                case SSL_Auto:
+                default:
+                        //FIXME: add new "set ssl" option for setting minimum version
+                        break;
+        }
+#endif
         SSL_CTX_set_default_verify_paths(C->ctx);
         if (CACertificateFile || CACertificatePath) {
                 if (! SSL_CTX_load_verify_locations(C->ctx, CACertificateFile, CACertificatePath)) {
@@ -693,7 +759,11 @@ SslServer_T SslServer_new(char *pemfile, char *clientpemfile, int socket) {
         S->pemfile = Str_dup(pemfile);
         if (clientpemfile)
                 S->clientpemfile = Str_dup(clientpemfile);
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
         const SSL_METHOD *method = SSLv23_server_method();
+#else
+        const SSL_METHOD *method = TLS_server_method();
+#endif
         if (! method) {
                 LogError("SSL: server method initialization failed -- %s\n", SSLERROR);
                 goto sslerror;
